import java.nio.channels.*;
import java.nio.file.*;
import javax.crypto.*;
import javax.crypto.spec.*;

public class FileEncrypter{

  public static void main(String[] args) throws Exception{
    var key = new SecretKeySpec("passwordpassword".getBytes(), "AES");
    var cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
    cipher.init(Cipher.ENCRYPT_MODE, key);

    var input = Paths.get(args[1]);
    var output = Paths.get(args[2]);

    if(args[0].equals("-c")){
      processWithDirectBuffer(input, output, cipher);
    }
    else{
      processWithStream(input, output, cipher);
    }

  }

  public static void processWithDirectBuffer(Path input, Path output, Cipher cipher) throws Exception{
    try(var src = FileChannel.open(input, StandardOpenOption.READ);
        var dst = FileChannel.open(output, StandardOpenOption.CREATE_NEW, StandardOpenOption.READ, StandardOpenOption.WRITE)){
      var srcMap = src.map(FileChannel.MapMode.READ_ONLY, 0, src.size());
      var dstMap = dst.map(FileChannel.MapMode.PRIVATE, 0, cipher.getOutputSize((int)src.size()));

      cipher.doFinal(srcMap, dstMap);
    }
  }

  public static void processWithStream(Path input, Path output, Cipher cipher) throws Exception{
    try(var src = Files.newInputStream(input);
        var dst = new CipherOutputStream(Files.newOutputStream(output, StandardOpenOption.CREATE_NEW, StandardOpenOption.WRITE), cipher)){
      src.transferTo(dst);
    }
  }

}
